const Parser = require('rss-parser');


class ListenerRss {
    name = undefined;
    address = undefined;
    timeloop = 5 * 60; // time in seconds
    customfields = [];

    // private fields
    parser = null;
    obj = null;
    loopRunning = false;

    constructor(info) {
        if(info !== undefined) {
            this.setData(info);
            this.setParser();
        }
    }

    setParser() {
        // set parser
        this.parser = new Parser(this.customfields !== undefined ? {
            customFields: {
                item: this.customfields.map((elt) => {
                    return Array.isArray(elt[1]) ? elt[1][0] : elt[1];
                })
            }
        } : undefined); // if customfield is set -> let's set the parser with, else let the option empty
    }

    setData(infos) {
        // Set data
        this.name = infos._name === undefined ? infos._address : infos._name; // if name is undefined let's take the address
        this.address = infos._address;
        this.timeloop = infos._timeloop;
        this.customfields = infos._customfields;
    }

    fetchRSS() {
        return this.parser.parseURL(this.address)
            .catch((err) => { throw new Error('bad address or no access : ' + err);});
    }

    /**
     * @brief call the callback function each looptime
     * @param callback function who's going to be called with the latest get
     */
    start(callback) {
        this.loopRunning = true;

        (async () => {
            while(this.loopRunning === true) {
                callback(await this.fetchRSS());
                await new Promise(res => setTimeout(res, this.timeloop * 1000));
            }
        })();
    }

    /**
     * @brief stop the async function
     */
    stop() {
        this.loopRunning = false;
    }
}

module.exports = ListenerRss